unit About;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, dlgAbout, StdCtrls;

Const
    csProduct  = 'ProductName';
    csCompany  = 'CompanyName';
    csAuthor   = 'Author';
    csVFITrans = '\VarFileInfo\Translation';
    csStrFInfo = '\StringFileInfo\';

type
	LongWord = Cardinal;
	TRegisterEvent = procedure( Sender : TObject; Var Accept : Boolean ) of object;

  TAboutDialog = class(TComponent)
  private
    { Private declarations }
    FFreeware : Boolean;
    FCaption  : String;
    FUserName : String;
    FRelsDate : String;
    FLicence  : TStrings;

    FProduct  : String;
    FCompany  : String;
    FAuthor   : String;
    FVersion  : LongWord;
    FRelease  : LongWord;

    FOnRegister : TRegisterEvent;

    Procedure GetVerInfo;
    Procedure SetLicence( ss : TStrings );

  protected
    { Protected declarations }
  public
    { Public declarations }
    Property Product  : String   read FProduct;
    Property Company  : String   read FCompany;
    Property Author   : String   read FAuthor;
    Property Version  : LongWord read FVersion;
    Property Release  : LongWord read FRelease;

	Constructor Create( AOwner : TComponent ); Override;
    Destructor  Destroy; Override;

    Function  Execute : Boolean;
	Procedure WebLink( aLabel : TLabel; aStr : String );
    Function  HyperLink( aStr : String ) : String;

  published
    { Published declarations }
    Property Freeware : Boolean read FFreeware write FFreeware default false;
    Property Caption  : String  read FCaption  write FCaption;
    Property UserName : String  read FUserName write FUserName;
    Property ReleaseDate : String  read FRelsDate write FRelsDate;
    Property Licence  : TStrings read FLicence write SetLicence;

  	Property OnRegister : TRegisterEvent read FOnRegister write FOnRegister;
  end;

procedure Register;

implementation

Procedure TAboutDialog.GetVerInfo;
Type
	PWord = ^Word;
Var
	dwSize, dwHandle, nLen : Cardinal;
    pBuf  : Pointer;
    pTemp : Pointer;
    pInfo : PVSFixedFileInfo;
    asLanguage : AnsiString;
    asQuery : AnsiString;
Begin
   	dwSize := GetFileVersionInfoSize( PChar( Application.ExeName ), dwHandle );
    If ( dwSize > 0 ) Then
    Begin
       	GetMem( pBuf, dwSize );
        Try
           	If ( GetFileVersionInfo( PChar( Application.ExeName ),
               	0, dwSize, PChar( pBuf ) ) ) Then
            Begin
               	VerQueryValue( pBuf, '\', Pointer( pInfo ), nLen );
                FVersion := pInfo^.dwFileVersionMS;
                FRelease := pInfo^.dwFileVersionLS;
                VerQueryValue( pBuf, csVFITrans, pTemp, nLen );
                asLanguage := csStrFInfo + Format( '%.04X', [PWord( pTemp )^] );
                Inc( PWord( pTemp ) );
                asLanguage := asLanguage + Format( '%.04X', [PWord( pTemp )^] ) + '\';
               	asQuery := asLanguage + csProduct;
                If ( VerQueryValue( pBuf, PChar( asQuery ), pTemp, nLen ) ) Then
                   	FProduct := String( PChar( pTemp ) );
               	asQuery := asLanguage + csCompany;
                If ( VerQueryValue( pBuf, PChar( asQuery ), pTemp, nLen ) ) Then
                   	FCompany := String( PChar( pTemp ) );
                asQuery := asLanguage + csAuthor;
                If ( VerQueryValue( pBuf, PChar( asQuery ), pTemp, nLen ) ) Then
                   	FAuthor := String( PChar( pTemp ) );
            End
            Else
            Begin
               	FVersion := 0;
                FRelease := 0;
            End
        Finally
           	FreeMem( pBuf );
        End;
    End;
End;

Procedure TAboutDialog.SetLicence( ss : TStrings );
Begin
	FLicence.Assign( ss );
End;

Constructor TAboutDialog.Create( AOwner : TComponent );
Begin
	Inherited Create( AOwner );
    FOnRegister := Nil;
    FFreeware := false;
    FRelsDate := '2000';
    FLicence := TStringList.Create;
    GetVerInfo;
End;

Destructor TAboutDialog.Destroy;
Begin
	Inherited Destroy;
	FLicence.Destroy;
End;

Function TAboutDialog.Execute : Boolean;
Var
	pBuf : Pointer;
    nResult : Integer;
Begin
	nResult := 80;
	If ( ( FUserName = '' ) AND ( ( FFreeware = True ) OR ( NOT Assigned( FOnRegister ) ) ) ) Then
    Begin
    	GetMem( pBuf, 80 );
    	GetUserName( PChar( pBuf ), LongWord( nResult ) );
        FUserName := String( PChar( pBuf ) );
        FreeMem( pBuf );
    End;

    AboutDlg := TAboutDlg.Create( Self );
    nResult := AboutDlg.ShowModal;
    AboutDlg.Destroy;
    If ( nResult <> mrOk ) Then
    	Result := True
    Else
    	Result := False;
End;

Procedure TAboutDialog.WebLink( aLabel : TLabel; aStr : String );
Const
	csHttp = 'http://';
    csMail = 'mailto:';
Var
	nsi, nei : Integer;
Begin
	nsi := Pos( '<', aStr );
    nei := Pos( '>', aStr );
    With aLabel Do
    If ( ( nsi > 0 ) AND ( ( nei > nsi ) OR ( nei = 0 ) ) ) Then
    Begin
        Caption := Copy( aStr, 1, nsi - 1 );
        Cursor := crHandPoint;
    	With Font Do
        Begin
        	Color := clBlue;
            Style := [fsUnderline];
        End;
        If ( nei = 0 ) Then
        	nei := Length( aStr ) + 1;
        Hint := LowerCase( Copy( aStr, nsi + 1, nei - nsi - 1 ) );
        nsi := Pos( csHttp, Hint );
        nei := Pos( csMail, Hint );
        If ( ( nsi <> 1 ) AND ( nei <> 1 ) ) Then
        Begin
        	nsi := Pos( '@', Hint );
        	If ( nsi = 0 ) Then
            	Hint := String( csHttp + Hint )
            Else
            	Hint := String( csMail + Hint );
        End;
        ShowHint := true;
    End
    Else
    Begin
        Caption := aStr;
        Cursor := crDefault;
    	With Font Do
        Begin
        	Color := clBlack;
            Style := [];
        End;
        ShowHint := false;
    End;
End;

Function TAboutDialog.HyperLink( aStr : String ) : String;
Const
	csHttp = 'http://';
    csMail = 'mailto:';
Var
	nsi, nei : Integer;
    s, sName, sHref : String;
Begin
	nsi := Pos( '<', aStr );
    nei := Pos( '>', aStr );
    If ( ( nsi > 0 ) AND ( ( nei > nsi ) OR ( nei = 0 ) ) ) Then
    Begin
        sName := Copy( aStr, 1, nsi - 1 );
        If ( nei = 0 ) Then
        	nei := Length( aStr ) + 1;
        sHref := LowerCase( Copy( aStr, nsi + 1, nei - nsi - 1 ) );
        nsi := Pos( csHttp, sHref );
        nei := Pos( csMail, sHref );
        If ( ( nsi <> 1 ) AND ( nei <> 1 ) ) Then
        Begin
        	nsi := Pos( '@', sHref );
        	If ( nsi = 0 ) Then
            	sHref := String( csHttp + sHref )
            Else
            	sHref := String( csMail + sHref );
        End;
        s := '<a href="' + sHref + '">' + sName + '</a>';
    End
    Else
    	s := aStr;
    Result := s;
End;

procedure Register;
begin
  RegisterComponents('MentalCtrls', [TAboutDialog]);
end;

end.
