//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "CoolBtn.h"
#pragma package(smart_init)

//---------------------------------------------------------------------------

static const int cnMaxBorderWidth = 8;
static const int cnMaxNumGlyphs   = 4;

//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck( TCoolBtn * )
{
	new TCoolBtn( NULL );
}
//---------------------------------------------------------------------------
//  class TCoolBtn
//---------------------------------------------------------------------------

__fastcall TCoolBtn::TCoolBtn( TComponent* Owner ) :
	TGraphicControl( Owner )
{
	ControlStyle << csOpaque;
	FGlyph = new Graphics::TBitmap( );
    SetBounds( 0, 0, 36, 36 );
    FAllowAllUp = false;
    FBorderWidth = 1;
    FDown = false;
    FFlat = false;
    FGroupIndex = 0;
    FLayout = blCaptionBottom;
    FNumGlyphs = 1;
    FTransparent = true;
    FMouseIn = false;
    ParentFont = True;
}
//---------------------------------------------------------------------------

__fastcall TCoolBtn::~TCoolBtn( )
{
	delete FGlyph;
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetAllowAllUp( bool aAllowAllUp )
{
	if ( aAllowAllUp != FAllowAllUp )
    {
    	FAllowAllUp = aAllowAllUp;
        UpdateExclusive( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetBorderWidth( int aBorderWidth )
{
	if ( ( aBorderWidth != FBorderWidth ) && ( aBorderWidth >= 0 )
    	&& ( aBorderWidth <= cnMaxBorderWidth ) )
    {
    	FBorderWidth = aBorderWidth;
        Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetText( TCaption aCaption )
{
	if ( aCaption != FCaption )
    {
    	FCaption = aCaption;
    	Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetDown( bool aDown )
{
	if ( aDown != FDown )
    {
    	FDown = aDown;
        if ( aDown && ( FGroupIndex > 0 ) )
        	UpdateExclusive( );
        Invalidate( );
    }
    else if ( aDown && FAllowAllUp )
    {
    	FDown = false;
        Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetFlat( bool aFlat )
{
	if ( aFlat != FFlat )
    {
    	FFlat = aFlat;
        if ( aFlat )
        	ControlStyle >> csOpaque;
        else
        	ControlStyle << csOpaque;
        Invalidate( );
	}
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetGroupIndex( int aGroupIndex )
{
	if ( ( aGroupIndex != FGroupIndex ) && ( aGroupIndex >= 0 ) )
    {
    	FGroupIndex = aGroupIndex;
        UpdateExclusive( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetGlyph( Graphics::TBitmap *aGlyph )
{
    if ( aGlyph && aGlyph->Width && aGlyph->Height )
    {
		FGlyph->Assign( aGlyph );
        int     n = aGlyph->Width / aGlyph->Height;
        if ( n < 1 )
        	n = 1;
        if ( n > cnMaxNumGlyphs )
        	n = cnMaxNumGlyphs;
        FNumGlyphs = n;
        n = FBorderWidth * 2;
    	Width  = aGlyph->Width / NumGlyphs + n;
        Height = aGlyph->Height + n;
    }
	Invalidate( );
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetLayout( CoolBtn::TButtonLayout aLayout )
{
	if ( aLayout != FLayout )
    {
    	FLayout = aLayout;
        Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetNumGlyphs( int aNumGlyphs )
{
	if ( aNumGlyphs < 0 )
    	aNumGlyphs = 1;
	if ( aNumGlyphs > cnMaxNumGlyphs )
    	aNumGlyphs = cnMaxNumGlyphs;
    if ( aNumGlyphs != FNumGlyphs )
    {
    	FNumGlyphs = aNumGlyphs;
    	Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::SetTransparent( bool aTransparent )
{
	if ( aTransparent != FGlyph->Transparent )
    {
		FGlyph->Transparent = aTransparent;
        Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::UpdateExclusive( void )
{
	TMessage    Msg;

	if ( FGroupIndex && Parent )
    {
    	Msg.Msg = CM_BUTTONPRESSED;
        Msg.WParam = FGroupIndex;
        Msg.LParam = ( int )this;
        Msg.Result = 0;
        Parent->Broadcast( &Msg );
    }
}
//---------------------------------------------------------------------------

bool __fastcall TCoolBtn::RectArrange( TRect &aGlyph, TRect &aCaption )
{
	if ( FGlyph->Empty && ( FCaption == "" ) )
    	return false;

    int     nW = Width - FBorderWidth * 2;
    int     nH = Height - FBorderWidth * 2;

    if ( FGlyph )
	    aGlyph   = Rect( 0, 0, FGlyph->Width / FNumGlyphs, FGlyph->Height );
    else
    	aGlyph   = Rect( 0, 0, 0, 0 );
    if ( FCaption == "" )
    	aCaption = Rect( 0, 0, 0, 0 );
    else
    	aCaption = Rect( 0, 0, Canvas->TextWidth( FCaption ), Canvas->TextHeight( FCaption ) );
    if ( nW > aGlyph.Right )
    	aGlyph.Left = ( nW - aGlyph.Right ) / 2;
    else
    	aGlyph.Right = nW;
    if ( nH > aGlyph.Bottom )
    	aGlyph.Top = ( nH - aGlyph.Bottom ) / 2;
    else
    	aGlyph.Bottom = nH;
    if ( nW > aCaption.Right )
    	aCaption.Left = ( nW - aCaption.Right ) / 2;
    else
    	aCaption.Right = nW;
    if ( nH > aCaption.Bottom )
    	aCaption.Top = ( nH - aCaption.Bottom ) / 2;
    else
    	aCaption.Bottom = nH;

    int     nMargins = ( FGlyph->Empty || ( FCaption == "" ) ) ? 2 : 3;
    int     nXMargin = ( nW - aGlyph.Right - aCaption.Right ) / nMargins;
    int     nYMargin = ( nH - aGlyph.Bottom - aCaption.Bottom ) / nMargins;
    if ( nXMargin < 0 )
    	nXMargin = 0;
    if ( nYMargin < 0 )
    	nYMargin = 0;

    switch ( FLayout ) {
    case blCaptionLeft :
        aGlyph.Left = nW - aGlyph.Right - nXMargin;
    	aCaption.Left = nXMargin;
    	break;
    case blCaptionTop :
    	aGlyph.Top = nH - aGlyph.Bottom - nYMargin;
        aCaption.Top = nYMargin;
    	break;
    case blCaptionRight :
    	aGlyph.Left = nXMargin;
        aCaption.Left = nW - aCaption.Right - nXMargin;
    	break;
    case blCaptionBottom :
    	aGlyph.Top = nYMargin;
        aCaption.Top = nH - aCaption.Bottom - nYMargin;
    	break;
    default :
    	break;
    }
    aGlyph.Left   += FBorderWidth;
    aGlyph.Top    += FBorderWidth;
    aGlyph.Right  += aGlyph.Left;
    aGlyph.Bottom += aGlyph.Top;
    aCaption.Left   += FBorderWidth;
    aCaption.Top    += FBorderWidth;
    aCaption.Right  += aCaption.Left;
    aCaption.Bottom += aCaption.Top;
    return true;
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::CMButtonPressed( TMessage &Msg )
{
	TCoolBtn   *pSender;

    if ( Msg.WParam == FGroupIndex )
    {
    	pSender = ( TCoolBtn * )Msg.LParam;
        if ( pSender != this )
        {
        	if ( ( pSender->Down ) && FDown )
            {
            	FDown = false;
                Invalidate( );
            }
            FAllowAllUp = pSender->AllowAllUp;
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::CMMouseEnter( TMessage &Msg )
{
	if ( FFlat && ( !FMouseIn ) && Enabled )
    {
    	FMouseIn = true;
        Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::CMMouseLeave( TMessage &Msg )
{
	if ( FFlat && FMouseIn && Enabled )
    {
    	FMouseIn = false;
        Invalidate( );
    }
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::MouseDown( TMouseButton Button, TShiftState Shift, int X, int Y )
{
	TGraphicControl::MouseDown( Button, Shift, X, Y );
    if ( ( Button == mbLeft ) && Enabled )
       	SetDown( true );
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::MouseUp( TMouseButton Button, TShiftState Shift, int X, int Y )
{
    TGraphicControl::MouseUp( Button, Shift, X, Y );
    if ( ( Button == mbLeft ) && Enabled && FDown && ( !FGroupIndex ) )
    	SetDown( false );
}
//---------------------------------------------------------------------------

void __fastcall TCoolBtn::Paint( void )
{
	TButtonState    nState;
    int             nBorder;

	if ( Enabled )
    {
    	if ( FDown )
        {
        	nState = bsDown;
            nBorder = -FBorderWidth;
        }
        else
        {
        	if ( !FFlat )
            {
            	nState = bsNormal;
                nBorder = FBorderWidth;
            }
            else if ( !FMouseIn )
            {
            	nState = bsNormal;
                nBorder = 0;
            }
            else
            {
            	nState = bsUp;
                nBorder = FBorderWidth;
            }
        }
	}
    else
    {
    	nState = bsDisabled;
        nBorder = ( !FFlat && FBorderWidth ) ? FBorderWidth : 0;
    }
	TRect   rt = Rect( 0, 0, Width, Height );
	// Initialize Brush-/Pen-settings
    Canvas->Brush->Color = Color;
    Canvas->Brush->Style = bsSolid;
    Canvas->Pen->Color = Color;
	// Draw background (face) of button
    Canvas->Rectangle( 0, 0, Width, Height );
	// Draw Frame3D in designtime so we can manage the button easily
    if ( ( !nBorder ) && ComponentState.Contains( csDesigning ) )
    	nBorder = 1;
    if ( nBorder < 0 )
   	   	Frame3D( Canvas, rt, clBtnShadow, clBtnHighlight, -nBorder );
    else if ( nBorder )
   		Frame3D( Canvas, rt, clBtnHighlight, clBtnShadow, nBorder );
   	TRect   rt1;
   	if ( !RectArrange( rt, rt1 ) )
    	return;
	if ( FGlyph && FGlyph->Width && FGlyph->Height )
    {
		int     nW = FGlyph->Width / FNumGlyphs;
    	int     nX = ( ( int )nState ) * nW;
		TRect   SrcRect = Rect( nX, 0, nX + nW, FGlyph->Height );
        TColor  TransColor = ( FTransparent ) ? FGlyph->TransparentColor : clBtnFace;
    	Canvas->BrushCopy( rt, FGlyph, SrcRect, TransColor );
    }
	if ( FCaption != "" )
    {
    	Canvas->Font->Assign( Font );
        Canvas->Pen->Color = Font->Color;
	    Canvas->TextOut( rt1.Left, rt1.Top, FCaption );
    }
}
//---------------------------------------------------------------------------

namespace Coolbtn
{
	void __fastcall PACKAGE Register( )
	{
		TComponentClass classes[1] = { __classid( TCoolBtn ) };
		RegisterComponents( "MentalCtrls", classes, 0 );
	}
}
//---------------------------------------------------------------------------
 