//---------------------------------------------------------------------------
#ifndef CoolBtnH
#define CoolBtnH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
//---------------------------------------------------------------------------

extern const int cnMaxBorderWidth;
extern const int cnMaxNumGlyphs;

//---------------------------------------------------------------------------

namespace CoolBtn {

typedef enum {
	bsNormal = 0, bsDisabled, bsDown, bsUp
} TButtonState;

typedef enum {
	blCaptionCenter, blCaptionLeft, blCaptionTop, blCaptionRight, blCaptionBottom
} TButtonLayout;

class PACKAGE TCoolBtn : public TGraphicControl
{
public :
	__fastcall TCoolBtn( TComponent* Owner );
    __fastcall ~TCoolBtn( );

private :
	bool    FAllowAllUp;
	int     FBorderWidth;
    TCaption    FCaption;
    bool    FDown;
    bool    FFlat;
    int     FGroupIndex;
    TButtonLayout   FLayout;
    int     FNumGlyphs;
    bool    FTransparent;
    Graphics::TBitmap  *FGlyph;
    bool    FMouseIn;   //  No published

    void    __fastcall  SetAllowAllUp( bool aAllowAllUp );
    void    __fastcall  SetBorderWidth( int aWidth );
    void    __fastcall  SetText( TCaption aCaption );
    void    __fastcall  SetDown( bool aDown );
    void    __fastcall  SetFlat( bool aFlat );
    void    __fastcall  SetGroupIndex( int aGroupIndex );
    void    __fastcall  SetLayout( TButtonLayout aLayout );
    void    __fastcall  SetNumGlyphs( int aNumGlyphs );
    void    __fastcall  SetTransparent( bool aTransparent );
    void    __fastcall  SetGlyph( Graphics::TBitmap *aGlyph );

    void    __fastcall  UpdateExclusive( void );
    bool    __fastcall  RectArrange( TRect &aGlyph, TRect &aCaption );
    
    MESSAGE void    __fastcall  CMButtonPressed( TMessage &Msg );
    MESSAGE void    __fastcall  CMMouseEnter( TMessage &Msg );
    MESSAGE void    __fastcall  CMMouseLeave( TMessage &Msg );

protected :
	DYNAMIC void    __fastcall  MouseDown( TMouseButton Button, TShiftState Shift, int X, int Y );
    DYNAMIC void    __fastcall  MouseUp( TMouseButton Button, TShiftState Shift, int X, int Y );
	virtual void    __fastcall  Paint( void );

__published :
    __property  bool    AllowAllUp = { read = FAllowAllUp, write = SetAllowAllUp, default = false };
    __property  int     BorderWidth = { read = FBorderWidth, write = SetBorderWidth, default = 1 };
    __property  TCaption        Caption = { read = FCaption, write = SetText };
    __property  Color;
    __property  bool    Down = { read = FDown, write = SetDown, default = false };
    __property  Enabled;
    __property  bool    Flat = { read = FFlat, write = SetFlat, default = false };
    __property  Font;
    __property  Graphics::TBitmap  *Glyph = { read = FGlyph, write = SetGlyph };
    __property  int     GroupIndex = { read = FGroupIndex, write = SetGroupIndex, default = 0 };
    __property  TButtonLayout   Layout = { read = FLayout, write = SetLayout, default = blCaptionBottom };
    __property  int     NumGlyphs = { read = FNumGlyphs, write = SetNumGlyphs, default = 1 };
	__property  Name;
    __property  ParentFont;
    __property  ParentShowHint;
    __property  ShowHint;
    __property  bool    Transparent = { read = FTransparent, write = SetTransparent, default = true };
    __property  Visible;
    __property  OnClick;
    __property  OnDblClick;
    __property  OnDragDrop;
    __property  OnDragOver;
    __property  OnEndDrag;
    __property  OnMouseDown;
    __property  OnMouseMove;
    __property  OnMouseUp;
    __property  OnStartDrag;
#pragma warn -inl
BEGIN_MESSAGE_MAP
    VCL_MESSAGE_HANDLER( CM_BUTTONPRESSED, TMessage, CMButtonPressed );
    VCL_MESSAGE_HANDLER( CM_MOUSEENTER, TMessage, CMMouseEnter );
    VCL_MESSAGE_HANDLER( CM_MOUSELEAVE, TMessage, CMMouseLeave );
END_MESSAGE_MAP( TGraphicControl );
#pragma warn .inl
};

} //end namespace CoolBtn

#if !defined(NO_IMPLICIT_NAMESPACE_USE)
using namespace CoolBtn;
#endif

//---------------------------------------------------------------------------
#endif

